/*******************************************************************************
 Rue's "is it alive" program for avr processors
  also makes a good skel to build your programs form.
*******************************************************************************/
 
/*



1     PB0 <- led
2	  PB1 
3	  PB2 
4	  PB3 
5	  PB4 
6     PB5
7     PB6 
8     PB7 
9     PA0 
10    PA1  	 
11    PA2 
12    PA3  
13    PA4   
14    PA5  
15    PA6	
16    PA7
17    PC7
18	  PC6
19	  PC5
20	  PC4
21	  PC3
22	  PC2
23	  PC1
24	  PC0
25	  PD7
26	  PD2
27	  PD3
28	  PD4
29	  PD5
30	  PD6


*/


#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>

#define OUTPUT             1
#define INPUT              0

// segment bit positions
#define A 0
#define B 1
#define C 2
#define D 3
#define E 4
#define F 5
#define G 6

/*

   AAAAA
  F     B
  F     B
  F     B
   GGGGG
  E     C
  E     C
  E     C
   DDDDD   dp


*/

// character genorator
unsigned char CG[] = { 
  (1<<A|1<<B|1<<C|1<<D|1<<E|1<<F),     /* 0 */
  (1<<B|1<<C),                         /* 1 */
  (1<<A|1<<B|1<<D|1<<E|1<<G),          /* 2 */
  (1<<A|1<<B|1<<C|1<<D|1<<G),          /* 3 */
  (1<<B|1<<C|1<<F|1<<G),               /* 4 */
  (1<<A|1<<C|1<<D|1<<F|1<<G),          /* 5 */
  (1<<C|1<<D|1<<E|1<<F|1<<G),          /* 6 */
  (1<<A|1<<B|1<<C),                    /* 7 */
  (1<<A|1<<B|1<<C|1<<D|1<<E|1<<F|1<<G),/* 8 */  
  (1<<A|1<<B|1<<C|1<<F|1<<G)           /* 9 */  
};



  #define SetBit(BIT, PORT)     PORT |= (1<<BIT)
  #define ClearBit(BIT, PORT)   PORT &= ~(1<<BIT)
  #define IsHigh(BIT, PORT)    (PORT & (1<<BIT)) != 0
  #define IsLow(BIT, PORT)     (PORT & (1<<BIT)) == 0
  #define NOP()                 asm volatile ("nop"::)

  // Bit positions
  #define Strobe             7
  #define Data               5
  #define Clock              6

  #define StrobeH()    SetBit(Strobe, PORTA)
  #define StrobeL()    ClearBit(Strobe, PORTA)
  #define DataH()      SetBit(Data, PORTA)
  #define DataL()      ClearBit(Data, PORTA)
  #define ClockH()     SetBit(Clock, PORTA)
  #define ClockL()     ClearBit(Clock, PORTA)
  #define ClockPulse() ClockH(); NOP(); ClockL()
  #define SendOne()    DataH(); ClockPulse()
  #define SendZero()   DataL(); ClockPulse()
  
  
  volatile unsigned int AdcValues[8];

void Delay(unsigned int delay);
void Delay2(unsigned int delay);
void Send (unsigned char bits) ;
void printNumber16(unsigned int n);

void AnalogInit ( void );
int  Analog     ( int n );



int main (void) {

  unsigned int i;

  // set up directions 
  DDRA = (INPUT << PA0 | INPUT << PA1 |INPUT << PA2 |INPUT << PA3 |INPUT << PA4 |OUTPUT << PA5 |OUTPUT << PA6 |OUTPUT << PA7);
  DDRB = (INPUT << PB0 | INPUT << PB1 |INPUT << PB2 |INPUT << PB3 |INPUT << PB4 |INPUT << PB5 |INPUT << PB6 |INPUT << PB7);
  DDRC = (INPUT << PC0 | INPUT << PC1 |INPUT << PC2 |INPUT << PC3 |INPUT << PC4 |INPUT << PC5 |INPUT << PC6 |INPUT << PC7);
  DDRD = (INPUT << PD0 | INPUT << PD1 |INPUT << PD2 |INPUT << PD3 |INPUT << PD4 |INPUT << PD5 |INPUT << PD6 |INPUT << PD7);        

  AnalogInit();
  sei();
  
  while(1){ 
     i = Analog(0);                  
     printNumber16((int)((float)i * 9.776)); // scale to 10.00
     //printNumber16((int)((float)i * 13.686));  // scale to 14.00
     
     Delay(32000);
   }
   
      
}



void Delay(unsigned int delay) {
  unsigned int x;
  for (x = delay; x != 0; x--) {
    asm volatile ("nop"::); 
  }
}
void Delay2(unsigned int delay) {
  unsigned int x;
  for (x = delay; x != 0; x--) {
    Delay(65000);
  }
}

void Send (unsigned char bits) {
  unsigned char temp;  
  for( temp = 128; temp != 0; temp = temp >> 1) {    
    if ( (bits & temp) == 0 ) {
      SendZero();
    } else {
      SendOne();
    }    
  } 
}

void printNumber16(unsigned int n) {
  unsigned int d;
  
  // we have no 10000's place, so screw it.
  
    
  d = n/10000;
  Send(~CG[d]);
  n -= d * 10000;
  
  d = n/1000;
  Send(~CG[d]);
  n -= d * 1000;
  
  d = n/100;
  Send(~CG[d]);
  n -= d * 100;
  
  d = n/10;
  Send(~CG[d]);
  n -= d * 10;
  
//  n -= d * 10;  // we drop the ones palce, effectivly dividing the value by 10, but in themeantime we get he accuracy from teh lost place.
//  Send(~CG[n]);
}




void AnalogInit (void) {

  int i;

  // Activate ADC with Prescaler 
  ADCSRA =  1 << ADEN  |
            1 << ADSC  | /* start a converstion, irq will take from here */
            0 << ADATE |
            0 << ADIF  |
            1 << ADIE  | /* enable interrupt */
            1 << ADPS2 |
            0 << ADPS1 |
            0 << ADPS0 ;
                        
            
  for (i = 0; i < 8; AdcValues[i++] = 0);
  
}

int Analog (int n) {
  return AdcValues[n & 7];
}

SIGNAL(SIG_ADC) {
  int i;
  
  i = ADMUX & 7;       // what channel we on?
  AdcValues[i] = ADC;  // save value
  i++;                 // next value
  ADMUX = i & 7;       // select channel
  ADCSRA |= _BV(ADSC); // start next conversion
  return;
}





